﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TestLibrary
{
    /// <summary>
    /// 커스텀 컨트롤
    /// </summary>
    public partial class CustomControl : Grid
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Delegate
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 커스텀 컨트롤 이벤트 핸들러 - CustomControlEventHandler(sender, e)

        /// <summary>
        /// 커스텀 컨트롤 이벤트 핸들러
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        public delegate void CustomControlEventHandler(object sender, CustomControlEventArgs e);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 버튼 클릭시 이벤트 - ButtonClick

        /// <summary>
        /// 버튼 클릭시 이벤트
        /// </summary>
        public event CustomControlEventHandler ButtonClick;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 배경 브러시
        /// </summary>
        private SolidColorBrush backgroundBrush;

        /// <summary>
        /// 전경 브러시
        /// </summary>
        private SolidColorBrush foregroundBrush;

        /// <summary>
        /// 폰트 패밀리
        /// </summary>
        private FontFamily fontFamily;

        /// <summary>
        /// 폰트 크기
        /// </summary>
        private double fontSize;

        /// <summary>
        /// 폰트 가중치
        /// </summary>
        private FontWeight fontWeight;

        /// <summary>
        /// 폰트 스타일
        /// </summary>
        private FontStyle fontStyle;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 배경 브러시 - BackgroundBrush

        /// <summary>
        /// 배경 브러시
        /// </summary>
        public SolidColorBrush BackgroundBrush
        {
            get
            {
                return this.backgroundBrush;
            }
            set
            {
                this.backgroundBrush = value;

                this.mainGrid.Background = value;
            }
        }

        #endregion
        #region 전경 브러시 - ForegroundBrush

        /// <summary>
        /// 전경 브러시
        /// </summary>
        public SolidColorBrush ForegroundBrush
        {
            get
            {
                return this.foregroundBrush;
            }
            set
            {
                this.foregroundBrush = value;

                this.nameTextBlock.Foreground    = value;
                this.addressTextBlock.Foreground = value;
                this.cityTextBlock.Foreground    = value;
                this.stateTextBlock.Foreground   = value;
                this.zipTextBlock.Foreground     = value;
            }
        }

        #endregion
        #region 폰트 패밀리 - FontFamily

        /// <summary>
        /// 폰트 패밀리
        /// </summary>
        public FontFamily FontFamily
        {
            get
            {
                return this.fontFamily;
            }
            set
            {
                this.fontFamily = value;

                this.nameTextBlock.FontFamily    = value;
                this.addressTextBlock.FontFamily = value;
                this.cityTextBlock.FontFamily    = value;
                this.stateTextBlock.FontFamily   = value;
                this.zipTextBlock.FontFamily     = value;
            }
        }

        #endregion
        #region 폰트 크기 - FontSize

        /// <summary>
        /// 폰트 크기
        /// </summary>
        public double FontSize
        {
            get
            {
                return this.fontSize;
            }
            set
            {
                this.fontSize = value;

                this.nameTextBlock.FontSize    = value;
                this.addressTextBlock.FontSize = value;
                this.cityTextBlock.FontSize    = value;
                this.stateTextBlock.FontSize   = value;
                this.zipTextBlock.FontSize     = value;
            }
        }

        #endregion
        #region 폰트 가중치 - FontWeight

        /// <summary>
        /// 폰트 가중치
        /// </summary>
        public FontWeight FontWeight
        {
            get
            {
                return this.fontWeight;
            }
            set
            {
                this.fontWeight = value;

                this.nameTextBlock.FontWeight    = value;
                this.addressTextBlock.FontWeight = value;
                this.cityTextBlock.FontWeight    = value;
                this.stateTextBlock.FontWeight   = value;
                this.zipTextBlock.FontWeight     = value;
            }
        }

        #endregion
        #region 폰트 스타일 - FontStyle

        /// <summary>
        /// 폰트 스타일
        /// </summary>
        public FontStyle FontStyle
        {
            get
            {
                return this.fontStyle;
            }
            set
            {
                this.fontStyle = value;

                this.nameTextBlock.FontStyle    = value;
                this.addressTextBlock.FontStyle = value;
                this.cityTextBlock.FontStyle    = value;
                this.stateTextBlock.FontStyle   = value;
                this.zipTextBlock.FontStyle     = value;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public CustomControl()
        {
            InitializeComponent();

            Loaded                  += Grid_Loaded;
            this.okButton.Click     += button_Click;
            this.cancelButton.Click += button_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 그리드 로드시 처리하기 - Grid_Loaded(sender, e)

        /// <summary>
        /// 그리드 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Grid_Loaded(object sender, EventArgs e)
        {
            this.backgroundBrush = (SolidColorBrush)this.mainGrid.Background;
            this.foregroundBrush = (SolidColorBrush)this.nameTextBlock.Foreground;
            this.fontFamily      = this.nameTextBlock.FontFamily;
            this.fontSize        = this.nameTextBlock.FontSize;
            this.fontWeight      = this.nameTextBlock.FontWeight;
            this.fontStyle       = this.nameTextBlock.FontStyle;
        }

        #endregion
        #region 버튼 클릭시 처리하기 - button_Click(sender, e)

        /// <summary>
        /// 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void button_Click(object sender, RoutedEventArgs e)
        {
            CustomControlEventArgs customControlEventArgs = new CustomControlEventArgs
            (
                true,
                this.nameTextBox.Text,
                this.addressTextBox.Text,
                this.cityTextBox.Text,
                this.stateTextBox.Text,
                this.zipTextBox.Text
            );

            if(sender == this.cancelButton)
            {
                customControlEventArgs.OKButtonClicked = false;
            }

            if(ButtonClick != null)
            {
                ButtonClick(this, customControlEventArgs);
            }
        }

        #endregion
    }
}